#include "Variant.h"
#include "VariantDict.h"

#include <boost/serialization/nvp.hpp>
#include <boost/serialization/vector.hpp>
#include <boost/serialization/shared_ptr.hpp>
#include <boost/archive/xml_iarchive.hpp>
#include <boost/archive/xml_oarchive.hpp>

Variant::Variant()
{
}

Variant::Variant(const VariantDictPtr & value)
{
	setVariantDict(value);
}

Variant::Variant(const std::vector<Variant> & value)
{
	setVariantVec(value);
}

Variant::Variant(const std::string & value)
{
	setString(value);
}

VariantDictPtr Variant::getVariantDict() const
{
	return _vdp;
}

void Variant::setVariantDict( const VariantDictPtr & value )
{
    _vdp = value;
    _type = 9;
}

const std::vector<Variant> & Variant::getVariantVec() const
{
	return _svv;
}

void Variant::setVariantVec( const std::vector<Variant> & value  )
{
    _svv = value;
	_type = 264;
}

std::string Variant::getString() const
{
    return _str;
}

void Variant::setString( const std::string & value  )
{
	_str = value;
    _type = 5;
}

template <typename Archive>
void save( Archive & ar, const Variant & obj, const unsigned int /*file_version*/ )
{
    int type = obj._type;
    ar & boost::serialization::make_nvp( "type", type );
    switch( type )
    {
    case 9:
        {
            const VariantDictPtr & v = obj.getVariantDict();
            ar & boost::serialization::make_nvp( "value", v );
        }
        break;
    case 264:
        {
            const std::vector<Variant>& v = obj.getVariantVec();
            ar & boost::serialization::make_nvp( "value", v );
        }
        break;
    case 5:
        {
            const std::string& v = obj.getString();
            ar & boost::serialization::make_nvp( "value", v );
        }
        break;

    default:
        throw new std::runtime_error( " not supported for serialization" );
        break;
    }
}

template <typename Archive>
void load( Archive & ar, Variant & obj, const unsigned int /*file_version*/ )
{
    int type;
    ar & boost::serialization::make_nvp( "type", type );
    switch( type )
    {
    case 9:
        {
			ar >> boost::serialization::make_nvp( "value", obj._vdp);
			obj._type = 9;
        }
        break;
    case 264:
        {
			ar >> boost::serialization::make_nvp( "value", obj._svv);
			obj._type = 264;
        }
        break;
    case 5:
        {
            ar & boost::serialization::make_nvp( "value", obj._str );
			obj._type = 5;
        }
        break;
    default:
        throw new std::runtime_error( " not supported for serialization" );
        break;
    }
}

template <typename Archive>
void serialize( Archive & ar, Variant & obj, const unsigned int file_version )
{
    boost::serialization::split_free(ar, obj, file_version);
}

template void serialize<boost::archive::xml_oarchive>( boost::archive::xml_oarchive& ar, Variant & obj, const unsigned int file_version );
template void serialize<boost::archive::xml_iarchive>( boost::archive::xml_iarchive& ar, Variant & obj, const unsigned int file_version );
