#ifndef VARIANT_H
#define VARIANT_H


#include <vector>
#include <boost/shared_ptr.hpp>
#include <boost/serialization/access.hpp>
#include <boost/archive/xml_iarchive.hpp>
#include <boost/archive/xml_oarchive.hpp>

class Variant;
typedef boost::shared_ptr<Variant> VariantPtr;

class VariantDict;
typedef boost::shared_ptr<VariantDict> VariantDictPtr;

class Variant
{
public:
	Variant();
	Variant(const VariantDictPtr & value);
	Variant(const std::vector<Variant> & value);
	Variant(const std::string & value);

    VariantDictPtr getVariantDict() const;
    void setVariantDict( const VariantDictPtr & value );

    const std::vector<Variant> & getVariantVec() const;
    void setVariantVec( const std::vector<Variant> & value  );

    std::string getString() const;
    void setString( const std::string & value  );

    double getDouble() const;
    void setDouble( const double & value  );

    int getInt() const;
    void setInt( int value );

    template <typename Archive>
    friend void serialize( Archive & ar, Variant & obj, const unsigned int file_version );

    template <typename Archive>
    friend void save( Archive & ar, const Variant & obj, const unsigned int file_version );

    template <typename Archive>
    friend void load( Archive & ar, Variant & obj, const unsigned int file_version );

    friend class boost::serialization::access;

private:
	VariantDictPtr _vdp;
	std::vector<Variant> _svv;
	std::string     _str;
    int             _type;
};

// Comment out this line to stop the crash
BOOST_CLASS_TRACKING(Variant, boost::serialization::track_always)

#endif
