// Copyright (c) 2000, 2002 - 2010 Citadel LLC, Confidential
// Information of Citadel LLC, or its affiliates.  All rights reserved.
// Copyright (c) 2000, 2002 - 2009 Citadel Investment Group, Confidential
// Information of Citadel Investment Group, or its affiliates.  All rights reserved.

#include "VariantDict.h"

#include <boost/serialization/nvp.hpp>
#include <boost/serialization/map.hpp>
#include <boost/serialization/vector.hpp>
#include <boost/archive/xml_iarchive.hpp>
#include <boost/archive/xml_oarchive.hpp>

VariantDict::VariantDict()
{
}

VariantDict::VariantDict( const std::vector< std::string >    & keys,
						  const std::vector< Variant > &        values )
{
	std::vector< Variant >::const_iterator valueIt = values.begin();
	for( std::vector< std::string >::const_iterator keyIt = keys.begin(), keyEnd = keys.end(); keyIt != keyEnd; ++keyIt, ++valueIt )
	{
		_variantMap.insert( std::make_pair(*keyIt, *valueIt) );
	}
}

Variant VariantDict::getValue( const std::string & key ) const
{
	VariantMap::const_iterator findIt = _variantMap.find( key );
	return findIt->second;
}

template <typename Archive>
void serialize( Archive & ar, VariantDict & obj, const unsigned int /* file_version */ )
{
	ar & boost::serialization::make_nvp( "_variantMap", obj._variantMap);
}

template void serialize<boost::archive::xml_oarchive>( boost::archive::xml_oarchive& ar, VariantDict & obj, const unsigned int file_version );
template void serialize<boost::archive::xml_iarchive>( boost::archive::xml_iarchive& ar, VariantDict & obj, const unsigned int file_version );
